angular.module('camotion')
    .controller('CamerasController', ['$scope', 'CamerasService', '$timeout',
        function($scope, CamerasService, $timeout) {

            $scope.query = "";
            $scope.maxPage = 1;
            $scope.page = 0;
            $scope.perPage = 6;
            $scope.pages = [];

            $scope.cameras = [];
            $scope.images = {};
            $scope.interval = 1000;

            $scope.playPause = function(camera)
            {
                if ($scope.isPlaying(camera)) {
                    $scope.pause(camera);
                }
                else {
                    $scope.play(camera);
                }
            };

            $scope.play = function(camera)
            {
                if (!$scope.isPlaying(camera)) {
                    $scope.images[camera.Id] = {
                        image: null,
                        timerId: 0
                    };
                    $scope.updateCamera(camera);
                }
            };

            $scope.pause = function(camera)
            {
                if ($scope.isPlaying(camera)) {
                    $scope.images[camera.Id] = null;
                }
            };

            $scope.isPlaying = function(camera)
            {
                return $scope.images[camera.Id] != null;
            };

            $scope.updateCamera = function(camera)
            {
                if ($scope.isPlaying(camera)) {
                    var promiseLoadCamera = {
                        id: "promiseLoadCamera_" + camera.Id,
                        errorGroups: ["camera_" + camera.Id]
                    };
                    CamerasService.getImage({camera_id: camera.Id}, promiseLoadCamera)
                        .then(function (image) {
                            if ($scope.isPlaying(camera)) {
                                $scope.images[camera.Id] = {
                                    image: image.Image,
                                    timerId: $timeout(function () {
                                        $scope.updateCamera(camera);
                                    }, $scope.interval)
                                };
                                angular.element('#camera_' + camera.Id)[0].src = "data:image/jpeg;base64," + image.Image;
                            }
                        }, function (error) {
                            $scope.pause(camera);
                        });
                }
            };

            $scope.loadPage = function(page)
            {
                if (page < 0 || page >= $scope.maxPage) {
                    return;
                }

                var promiseLoadCameras = {
                    id: "promiseLoadCameras",
                    groups: ["camerasView"]
                };
                $scope.images = {};

                CamerasService.getAll({page: page, perPage: $scope.perPage, query: $scope.query}, promiseLoadCameras)
                    .then(function(cameras) {
                        $scope.page = page;
                        $scope.images = {};
                        $scope.cameras = cameras.Data;
                        $scope.pages = [];
                        $scope.maxPage = Math.max(cameras.Count / $scope.perPage, 1);
                        var start = Math.max(0, $scope.page - 5);
                        var end = Math.min(start + 10, $scope.maxPage);
                        for (var i = start; i < end; ++i) {
                            $scope.pages.push(i);
                        }

                        for (i = 0; i < $scope.cameras.length; ++i) {
                            var camera = angular.copy($scope.cameras[i]);
                            $scope.play(camera);
                        }
                    });
            };

            $scope.$on('$destroy', function(){
                $scope.images = {};
            });

            $scope.loadPage($scope.page);
    }]);
